@@ -1,9 +1,8 @@
 {-# LANGUAGE ConstraintKinds #-}
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE MultiWayIf #-}
 
-module MainSpec (main, spec) where
+module MainSpec where
 
 import           Test.Hspec
 import           Test.HUnit (assertEqual, Assertion)
